@@ -22,12 +22,12 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+from enum import IntEnum
 from gettext import gettext as _
+
 from gi.repository import Gdk, GObject, Gtk
 
-from gnomemusic import log
 from gnomemusic.player import PlayerPlaylist
-from gnomemusic.utils import View
 from gnomemusic.search import Search
 from gnomemusic.widgets.albumcover import AlbumCover
 from gnomemusic.widgets.albumwidget import AlbumWidget
@@ -46,9 +46,18 @@ class SearchView(Gtk.Stack):
 
     __gtype_name__ = "SearchView"
 
+    class State(IntEnum):
+        """The different states of SearchView
+        """
+        MAIN = 0
+        ALL_ALBUMS = 1
+        ALL_ARTISTS = 2
+        ALBUM = 3
+        ARTIST = 4
+
     search_state = GObject.Property(type=int, default=Search.State.NONE)
-    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
+    state = GObject.Property(type=int, default=State.MAIN)
 
     _album_header = Gtk.Template.Child()
     _album_flowbox = Gtk.Template.Child()
@@ -63,11 +72,7 @@ class SearchView(Gtk.Stack):
     _view_all_albums = Gtk.Template.Child()
     _view_all_artists = Gtk.Template.Child()
 
-    def __repr__(self):
-        return '<SearchView>'
-
-    @log
-    def __init__(self, application, player=None):
+    def __init__(self, application):
         """Initialize SearchView
 
         :param GtkApplication application: The Application object
@@ -123,7 +128,7 @@ class SearchView(Gtk.Stack):
             'selection-mode', self._window, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL)
 
-        self._album_widget = AlbumWidget(player, self)
+        self._album_widget = AlbumWidget(self._application)
         self._album_widget.bind_property(
             "selection-mode", self, "selection-mode",
             GObject.BindingFlags.BIDIRECTIONAL)
@@ -133,7 +138,6 @@ class SearchView(Gtk.Stack):
         self._scrolled_artist_window = None
 
         self._search_mode_active = False
-        # self.connect("notify::search-state", self._on_search_state_changed)
 
     def _core_filter(self, coreitem, coremodel, nr_items):
         if coremodel.get_n_items() <= 5:
@@ -160,6 +164,7 @@ class SearchView(Gtk.Stack):
 
     def _create_album_widget(self, corealbum):
         album_widget = AlbumCover(corealbum)
+        album_widget.retrieve()
 
         self.bind_property(
             "selection-mode", album_widget, "selection-mode",
@@ -168,7 +173,7 @@ class SearchView(Gtk.Stack):
 
         # NOTE: Adding SYNC_CREATE here will trigger all the nested
         # models to be created. This will slow down initial start,
-        # but will improve initial 'selecte all' speed.
+        # but will improve initial 'select all' speed.
         album_widget.bind_property(
             "selected", corealbum, "selected",
             GObject.BindingFlags.BIDIRECTIONAL)
@@ -324,6 +329,7 @@ class SearchView(Gtk.Stack):
         # Update and display the album widget if not in selection mode
         self._album_widget.update(corealbum)
 
+        self.props.state = SearchView.State.ALBUM
         self._headerbar.props.state = HeaderBar.State.SEARCH
         self._headerbar.props.title = corealbum.props.title
         self._headerbar.props.subtitle = corealbum.props.artist
@@ -338,7 +344,7 @@ class SearchView(Gtk.Stack):
             return
 
         artist_albums_widget = ArtistAlbumsWidget(
-            coreartist, self._application, False)
+            coreartist, self._application, True)
         # FIXME: Recreating a view here. Alternate solution is used
         # in AlbumsView: one view created and an update function.
         # Settle on one design.
@@ -352,6 +358,7 @@ class SearchView(Gtk.Stack):
             "selection-mode", artist_albums_widget, "selection-mode",
             GObject.BindingFlags.BIDIRECTIONAL)
 
+        self.props.state = SearchView.State.ARTIST
         self._headerbar.props.state = HeaderBar.State.SEARCH
         self._headerbar.props.title = coreartist.props.artist
         self._headerbar.props.subtitle = None
@@ -361,6 +368,7 @@ class SearchView(Gtk.Stack):
 
     @Gtk.Template.Callback()
     def _on_all_artists_clicked(self, widget, event, user_data=None):
+        self.props.state = SearchView.State.ALL_ARTISTS
         self._headerbar.props.state = HeaderBar.State.SEARCH
         self._headerbar.props.title = _("Artists Results")
         self._headerbar.props.subtitle = None
@@ -375,6 +383,7 @@ class SearchView(Gtk.Stack):
 
     @Gtk.Template.Callback()
     def _on_all_albums_clicked(self, widget, event, user_data=None):
+        self.props.state = SearchView.State.ALL_ALBUMS
         self._headerbar.props.state = HeaderBar.State.SEARCH
         self._headerbar.props.title = _("Albums Results")
         self._headerbar.props.subtitle = None
@@ -388,53 +397,60 @@ class SearchView(Gtk.Stack):
         self.props.search_mode_active = False
 
     def _select_all(self, value):
-        with self._model.freeze_notify():
-            def song_select(child):
-                song_widget = child.get_child()
-                song_widget.props.selected = value
+        def child_select(child):
+            child.props.selected = value
 
-            def album_select(child):
-                child.props.selected = value
+        if self.props.state == SearchView.State.MAIN:
+            with self._model.freeze_notify():
+                def song_select(child):
+                    song_widget = child.get_child()
+                    song_widget.props.coresong.props.selected = value
 
-            def artist_select(child):
-                child.props.selected = value
-
-            self._songs_listbox.foreach(song_select)
-            self._album_flowbox.foreach(album_select)
-            self._artist_flowbox.foreach(artist_select)
+                self._songs_listbox.foreach(song_select)
+                self._album_flowbox.foreach(child_select)
+                self._artist_flowbox.foreach(child_select)
+        elif self.props.state == SearchView.State.ALL_ALBUMS:
+            with self._model.freeze_notify():
+                self._album_all_flowbox.foreach(child_select)
+        elif self.props.state == SearchView.State.ALL_ARTISTS:
+            with self._model.freeze_notify():
+                self._artist_all_flowbox.foreach(child_select)
+        elif self.props.state == SearchView.State.ALBUM:
+            view = self.get_visible_child()
+            if value is True:
+                view.select_all()
+            else:
+                view.deselect_all()
+        elif self.props.state == SearchView.State.ARTIST:
+            view = self.get_visible_child().get_child().get_child()
+            if value is True:
+                view.select_all()
+            else:
+                view.deselect_all()
 
     def select_all(self):
         self._select_all(True)
 
-    def unselect_all(self):
+    def deselect_all(self):
         self._select_all(False)
 
-    @log
     def _back_button_clicked(self, widget, data=None):
-        if self.get_visible_child() == self._scrolled_artist_window:
+        if self.get_visible_child() == self._search_results:
+            return
+        elif self.get_visible_child() == self._scrolled_artist_window:
             self.remove(self._scrolled_artist_window)
             self._scrolled_artist_window.destroy()
             self._scrolled_artist_window = None
-        elif self.get_visible_child() == self._search_results:
-            self._window.views[View.ALBUM].set_visible_child(
-                self._window.views[View.ALBUM]._grid)
 
         self.set_visible_child(self._search_results)
         self.props.search_mode_active = True
+        self.props.state = SearchView.State.MAIN
         self._headerbar.props.state = HeaderBar.State.MAIN
 
-    @log
     def _on_selection_mode_changed(self, widget, data=None):
-        if not self.props.selection_mode:
-            self.unselect_all()
-
-    @log
-    def _on_search_state_changed(self, klass, param):
-        # If a search is triggered when selection mode is activated,
-        # reset the number of selected items.
-        if (self.props.selection_mode
-                and self.props.search_state != Search.State.NONE):
-            self.props.selected_items_count = 0
+        if (not self.props.selection_mode
+                and self.get_parent().get_visible_child() == self):
+            self.deselect_all()
 
     @GObject.Property(type=bool, default=False)
     def search_mode_active(self):
